/*
 * Copyright (C) 2005 Universitat d'Alacant / Universidad de Alicante
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef _PATTERNLIST_
#define _PATTERNLIST_

#include <lttoolbox/Alphabet.H>
#include <lttoolbox/MatchExe.H>
#include <lttoolbox/Transducer.H>

#include <list>
#include <map>
#include <string>
#include <vector>

using namespace std;

typedef multimap<int, vector<int> > PatternStore;
typedef pair<PatternStore::iterator, PatternStore::iterator> PatternRange;

class PatternList
{
private:
  Alphabet alphabet;  
  PatternStore patterns;
  bool sequence;
  list<vector<int> > sequence_data;
  Transducer transducer;
  map<int, int> final_type;
  int sequence_id;

  void copy(PatternList const &o);
  void destroy();
  void insertOutOfSequence(string const &lemma, string const &tags,
			   vector<int> &result);
  void insertIntoSequence(int const id, string const &lemma, 
			  string const &tags);

  static int tagCount(string const &tags);
  static string tagAt(string const &tags, int const index);
  
public:
  static string const ANY_CHAR;
  static string const ANY_TAG;
  static string const QUEUE;

  PatternList();
  ~PatternList();
  PatternList(PatternList const &o);
  PatternList & operator =(PatternList const &o);

  void beginSequence();
  void endSequence();
  void insert(int const id, string const &lemma, string const &tags);
  void insert(int const id, int const otherid);
  PatternStore const & getPatterns();
  void buildTransducer();
  void print() const;
  void read(FILE *input);
  void write(FILE *output);
  
  MatchExe * newMatchExe();
  Alphabet & getAlphabet();
};

#endif
